# Code cited in Nakagawa, S. and Cuthill, I.C. 2007. 
# Effect size, confidence interval and statistical significance: 
# a practical guide for biologists. Biological Reviews, 82, pp. 591605.
# A concise defintion and the difference between Simulation and Boostrap
# is described in Faraway (2005,p.102-103)
# This script aims to present an example for both Simulation and Boostrap CI
# Please also refer to relavant sections in Faraway (2006)
# and Gelman & Hill (2007).
# Ee use simulated data (see below); we have predictors x1 and x2 and response y.
# We'd also like to obtain CIs for coefficients, b0 (intercept), b1 (x1) & b2 (x2)
# (see Gelman & Hill (2007) for the power of 'fake' data!)

x1<-1:100
x2<-ceiling(runif(100)*100)
b0<-10
b1<-2.1
b2<-3.2
sigma<-111 # residual error
n<-length(x1) # or length(x2)which is 100
y<-b0+b1*x1+b2*x2+rnorm(n,0, sigma) #normal error is assumed

# this is our data to use - it changes everytime you run
# the methods below can be employed for glm, lmer, lme etc....

# calculating CIs using Simulation#############

model<-lm(y~x1+x2) # model using an orignial set of data
n.sims<- 1000 # no of simulations
results<-array(NA,c(n.sims,3))# array or matrix to store coefficients (b0-2)
colnames(results)<-c("b0","b1","b2")
for (i in 1:1000){
  y.sim<-unlist(simulate(model))# generate y by simulation
  model.sim<-lm(y.sim~x1+x2)
  results[i,]<-coef(model.sim) # storing b0-2 from simulations
}

apply(results,2,mean)# get estimates of b0-2 from simulations
apply(results,2,median)# medians should be the same as means
apply(results,2,quantile,prob=c(0.025,0.975))# and 95% CIs
#####################################################

# calculating CIs using Boostrap ################
# here we use normally distributed data (y)
# but bootstrap deals with non-normal data too
# this is one kind of bootstrap.
# For more details, see Davison & Henkley (1997)and Manly (2007)

model<-lm(y~x1+x2) # model using an orignial set of data
n.sims<- 1000 # no of simulations
results<-array(NA,c(n.sims,3))# array or matrix to store coefficients (b0-2)
colnames(results)<-c("b0","b1","b2")
for (i in 1:1000){
  y.boot<-predict(model)+residuals(model)[sample(n,rep=T)]
  # generate y by bootsrap - shaffling residuals
  model.boot<-lm(y.boot~x1+x2)
  results[i,]<-coef(model.boot) # storing b0-2 from simulations
}

apply(results,2,mean)# get estimates of b0-2 from simulations
apply(results,2,median)# meidans are better estimates with non-normal data
apply(results,2,quantile,prob=c(0.025,0.975))# and 95% CIs
#########################################################

# download the package 'arms' - then you can use the function sim()
# which will get you simulated CIs for predictors artomatically!
# also check the package 'boot' (already installed)
# the end -  please email itchyshin@yahoo.co.nz
# if you find some bugs in the code - thanks






